#include "types.h"
#include "riscv.h"
#include "defs.h"
#include "param.h"
#include "memlayout.h"
#include "spinlock.h"
#include "proc.h"

uint64
sys_exit(void)
{
  int n;
  argint(0, &n);
  exit(n);
  return 0;  // not reached
}

uint64
sys_getpid(void)
{
  return myproc()->pid;
}

uint64
sys_fork(void)
{
  return fork();
}

uint64
sys_wait(void)
{
  uint64 p;
  argaddr(0, &p);
  return wait(p);
}

uint64
sys_sbrk(void)
{
  uint64 addr;
  int n;

  argint(0, &n);
  addr = myproc()->sz;
  if(growproc(n) < 0)
    return -1;
  return addr;
}

uint64
sys_sleep(void)
{
  int n;
  uint ticks0;

  argint(0, &n);
  if(n < 0)
    n = 0;
  acquire(&tickslock);
  ticks0 = ticks;
  while(ticks - ticks0 < n){
    if(killed(myproc())){
      release(&tickslock);
      return -1;
    }
    sleep(&ticks, &tickslock);
  }
  release(&tickslock);
  return 0;
}

uint64
sys_kill(void)
{
  int pid;

  argint(0, &pid);
  return kill(pid);
}

// return how many clock tick interrupts have occurred
// since start.
uint64
sys_uptime(void)
{
  uint xticks;

  acquire(&tickslock);
  xticks = ticks;
  release(&tickslock);
  return xticks;
}

uint64
sys_hello(void)
{
  printf("Hello, xv6!\n");
  return 0;
}

uint64
sys_ps(void)
{
  struct proc *p;
  printf("PID\tState\tName\n");
  for(p = proc; p < &proc[NPROC]; p++){
    if(p->state != UNUSED){
      printf("%d\t%d\t%s\n", p->pid, p->state, p->name);
    }
  }
  return 0;
}

uint64
sys_shutdown(void)
{

  // 同步文件系统
  begin_op();
  end_op();
  
  // 尝试优雅关机
  volatile uint32 *ctrl = (uint32*)SHUTDOWN_CTRL;
  volatile uint32 *addr = (uint32*)SHUTDOWN_ADDR;
  
  *ctrl = 1;
  __sync_synchronize();
  *addr = 0x5555;

  printf("Shutdown failed\n");
  
  // 如果直接关机失败,进入死循环
  for(;;) {
    asm volatile("wfi");
  }
  
  return 0;  // 不会执行到这里
}

uint64 
sys_top(void)
{
  struct proc *p;
  uint64 total_mem = PHYSTOP - KERNBASE;
  uint64 used_mem = 0;
  uint64 current_ticks;
  
  // 获取当前ticks
  acquire(&tickslock);
  current_ticks = ticks;
  release(&tickslock);
  // 清屏并移动光标
  printf("\x1b[2J\x1b[H");
  
  printf("=== XV6 TOP ===\n");
  // 系统信息
  printf("Uptime: %ld seconds\n", current_ticks/10);
  
  // 表头
  printf("PID\tCPU%%\tMEM\tTIME\tPRI\tSTATE\t\tNAME\n");
  
  // 计算内存使用
  for(p = proc; p < &proc[NPROC]; p++) {
    if(p->state != UNUSED) {
      used_mem += p->sz;
    }
  }
  
  // 遍历进程
  for(p = proc; p < &proc[NPROC]; p++) {
    if(p->state != UNUSED) {
      // CPU使用率计算(简化版)
      int cpu_usage = (p->cputicks * 100) / (ticks - p->startticks + 1);
      
      // 内存使用百分比
      int mem_percent = (p->sz * 100) / total_mem;
      
      // 运行时间(秒)
      int runtime = (ticks - p->startticks) / 10;
      
      // 状态字符串
      char *state;
      switch(p->state) {
        case SLEEPING: state = "sleep "; break;
        case RUNNABLE: state = "runble"; break;
        case RUNNING: state = "run   "; break;
        case ZOMBIE: state = "zombie"; break;
        default: state = "???   ";
      }
      
      printf("%d\t%d%%\t%d%%\t%d\t%d\t%s\t%s\n",
             p->pid, cpu_usage, mem_percent, runtime,
             p->priority, state, p->name);
    }
  }
  
  return 0;
}